; ---------------------------------------------------
; module description : 
; module file name   : Ignition.ASM
; created            : 11/28/97 7:41:22 AM
; ---------------------------------------------------
;
;      Q0      Q1      Q2      Q3      Q4
; --------------------------------------
; Cyl1 |       |       |FFFFFFF|    XX |
; --------------------------------------
; Cyl2 |    XX |       |       |FFFFFFF|
; --------------------------------------
; Cyl3 |FFFFFF |    XX |       |       |
; --------------------------------------
; Cyl4 |       |FFFFFF |    XX |       |
; --------------------------------------
;
; XX     : ignition pulse
; FFFFFF : iuel injector pulse
;
; time    crankshaft position   camshaft position
;  Q0          0 degrees            0 degrees (camshaft pulse)
;  Q1        180 degrees           90 degrees
;  Q2        360 degrees          180 degrees
;  Q3        540 degrees          270 degrees
;  Q4        720 degrees          360 degrees
;
; ---------------------------------------------------
; SetIgnitionTimes loads the compare registers
; The ignition begin and end times are saved in
; memory variables.  Each variable is saved in two
; words, collectively constituting a 32-bit time.
; These values are later loaded to the compare
; registers to trigger interrupts, which determine
; the ignition pulse begin and end times.
; ---------------------------------------------------
SetIgnitionTimes:
      push    r0
      push    r1
      calla   cc_UC, GetIgnitionAdvance
      mov     r1, r0
      calla   cc_UC, GetIgnitionDwell

; --- compute ignition 1 begin/end times ---
      mov     r2, CamPeriod
      mov     Ign1_End, r2
      sub     Ign1_End, r1

      mov     r2, Ign1_End
      mov     Ign1_Begin, r2
      sub     Ign1_Begin, r0

; --- compute ignition 2 start/end times ---
      mov     r2, Q1
      mov     Ign2_End, r2
      sub     Ign2_End, r1

      mov     r2, Ign2_End
      mov     Ign2_Begin, r2
      sub     Ign2_Begin, r0

; --- compute ignition 3 start/end times ---
      mov     r2, Q2
      mov     Ign3_End, r2
      sub     Ign3_End, r1

      mov     r2, Ign3_End
      mov     Ign3_Begin, r2
      sub     Ign3_Begin, r0

; --- compute ignition 4 start/end times ---
      mov     r2, Q3
      mov     Ign4_End, r2
      sub     Ign4_End, r1

      mov     r2, Ign4_End
      mov     Ign4_Begin, r2
      sub     Ign4_Begin, r0

; --- load compare registers ---
      mov     CC1, Ign1_Begin
      mov     CC2, Ign2_Begin
      mov     CC3, Ign3_Begin
      mov     CC4, Ign4_Begin

; --- all spark plugs off ---
      bset    IGN1
      bset    IGN2
      bset    IGN3
      bset    IGN4
      pop     r1
      pop     r0
      ret

; ---------------------------------------------------
; CC1 to CC4 interrupt service routines (ISRs)
;
; Each ISR is responsible for one cylinder.  The 
; ignition pulse begin times were loaded by the
; routine SetIgnitionTimes.  The interrupt when
; Timer0 reaches the begin time is handled first.
; The ignition pulse is activated (active low).
; The compare register is loaded with the ignition
; end time, read from memory (previously calculated
; by the routine SetIgnitionTimes).  The next compare
; match occurs at the end of the ignition pulse.
; The end of the ignition pulse also determines the 
; firing of the spark plug.
; ---------------------------------------------------

; ---------------------------------------------------
; CC2 interrupt service routine
; ---------------------------------------------------
ISR_CC1:
      push    r0                 ; save registers on stack

      cmp     CC1, Ign1_Begin
      jmpr    cc_NE, isrIgn101
      bclr    IGN1               ; start ignition dwell
      mov     CC1, Ign1_End      ; reload CC1
      pop     r0
      reti

isrIgn101:
      cmp     CC1, Ign1_End
      jmpr    cc_NE, isrIgn102
      bset    IGN1               ; end ignition dwell -- fire
      mov     CC1, Ign1_Begin    ; reload CC1
isrIgn102:
      pop     r0
      reti

; ---------------------------------------------------
; CC2 interrupt service routine
; ---------------------------------------------------
ISR_CC2:
      push    r0                 ; save registers on stack

      cmp     CC2, Ign2_Begin
      jmpr    cc_NE, isrIgn201
      bclr    IGN2               ; start ignition dwell
      mov     CC2, Ign2_End      ; reload CC2
      pop     r0
      reti

isrIgn201:
      cmp     CC2, Ign2_End
      jmpr    cc_NE, isrIgn202
      bset    IGN2               ; end ignition dwell -- fire
      mov     CC2, Ign2_Begin    ; reload CC2
isrIgn202:
      pop     r0
      reti

; ---------------------------------------------------
; CC3 interrupt service routine
; ---------------------------------------------------
ISR_CC3:
      push    r0                 ; save registers on stack

      cmp     CC3, Ign3_Begin
      jmpr    cc_NE, isrIgn301
      bclr    IGN3               ; start ignition dwell
      mov     CC3, Ign3_End      ; reload CC3
      pop     r0
      reti

isrIgn301:
      cmp     CC3, Ign3_End
      jmpr    cc_NE, isrIgn302
      bset    IGN3               ; end ignition dwell -- fire
      mov     CC3, Ign3_Begin    ; reload CC3
isrIgn302:
      pop     r0
      reti

; ---------------------------------------------------
; CC4 interrupt service routine
; ---------------------------------------------------
ISR_CC4:
      push    r0                 ; save registers on stack

      cmp     CC4, Ign4_Begin
      jmpr    cc_NE, isrIgn401
      bclr    IGN4               ; start ignition dwell
      mov     CC4, Ign4_End      ; reload CC4
      pop     r0
      reti

isrIgn401:
      cmp     CC4, Ign4_End
      jmpr    cc_NE, isrIgn402
      bset    IGN4               ; end ignition dwell -- fire
      mov     CC4, Ign4_Begin    ; reload CC4
isrIgn402:
      pop     r0
      reti

; ---------------------------------------------------
GetIgnitionAdvance:
      mov     r0, #10h
      ret
GetIgnitionDwell:
      mov     r0, #20h
      ret
; ---------------------------------------------------
; end of module...

