; ---------------------------------------------------
; module description : 
; module file name   : Main.ASM
; created            : 11/28/97 6:29:19 AM
; ---------------------------------------------------
; constants:
  MONITOR  equ  0C000h

; --- ignition outputs ---
IGN1 defb P2.1
IGN2 defb P2.2
IGN3 defb P2.3
IGN4 defb P2.4

; ---------------------------------------------------
; interrupt vectors:

     org      040h            ; CC0 interrupt vector
     jmpa     cc_UC, ISR_CC0

     org      044h            ; CC1 interrupt vector
     jmpa     cc_UC, ISR_CC1
     org      048h            ; CC2 interrupt vector
     jmpa     cc_UC, ISR_CC2
     org      04Ch            ; CC3 interrupt vector
     jmpa     cc_UC, ISR_CC3
     org      050h            ; CC4 interrupt vector
     jmpa     cc_UC, ISR_CC4

     org      098h            ; T6 interrupt vector
     jmpa     cc_UC, ISR_T6
     org      09Ch            ; GPT2 CAPREL interrupt vector
     jmpa     cc_UC, ISR_CAPREL
; ---------------------------------------------------

; --- run the program with the G200 command ---

      org     0200h     
MAIN: 
; --- set up I/O ports ---
      bclr    DP3.2           ; P3.2 input - CrankPulse
      bset    DP2.5           ; P2.5 output - tick indicator

      bclr    DP2.0           ; P2.0 input - CamPulse
      
      bset    DP2.1           ; P2.1 output - ignition 1
      bset    DP2.2           ; P2.2 output - ignition 2
      bset    DP2.3           ; P2.3 output - ignition 3
      bset    DP2.4           ; P2.4 output - ignition 4

; --- set up CAPCOM CC0 register ---
;      mov     CCM0, #01       ; CC0 capture at rising edge
      mov     CCM0, #02       ; CC0 capture at falling edge
; --- set up CAPCOM interrupt control registers ---
      mov     CC0IC, #48h     ; priority 2, group level 0
      mov     CC1IC, #47h     ; priority 1, group level 3
      mov     CC2IC, #46h     ; priority 1, group level 2
      mov     CC3IC, #45h     ; priority 1, group level 1
      mov     CC4IC, #44h     ; priority 1, group level 0

      or      CCM0, #4440h
      mov     CCM1, #4

; --- set up Timers 5 and 6 ---
; --- set up the timer control register ---
;      mov     T5CON, #0E003h  ; prescalar=32, Timer5 off
;      mov     T6CON, #08080h  ; prescalar= 4, Timer6 off
      mov     T5CON, #0E007h  ; prescalar=512, Timer5 off
      mov     T6CON, #08084h  ; prescalar= 64, Timer6 off

; --- set up interrupt control registers CRIC and T6IC ---
      mov     CRIC, #4Ch      ; priority 3, group level 1
      mov     T6IC, #4Dh      ; priority 3, group level 0
      bset    T5R             ; start timer 5
      bset    T6R             ; start timer 6

; --- set up Timer0 ---
; --- set up the timer control register ---
      mov     T01CON, #8      ; count GPT2 T6 underflows
      bset    T0R

; --- enable interrupts ---
      bset    IEN             ; enable interrupts

; --- foreground loop does the serial communications ---
LOOP:
      calla   cc_UC, getchr   ; get single-letter command
      bclr    r0.5            ; make upper case
      cmp     r0, #'D'
      jmpa    cc_NE, Loop01
      calla   cc_UC, Dump     ; 'D' for Dump
      jmpa    cc_UC, LOOP
Loop01:
      cmp     r0, #'Q'
      jmpa    cc_EQ, Quit     ; 'Q' for quit
      jmpa    cc_UC, LOOP

; --- clean up and quit the program ---
QUIT:
      mov     CC0IC, ZEROS
      mov     CC1IC, ZEROS
      mov     CC2IC, ZEROS
      mov     CC3IC, ZEROS
      mov     CC4IC, ZEROS
      mov     T0IC, ZEROS
      mov     T5IC, ZEROS
      mov     T6IC, ZEROS

      bclr    T0R             ; stop Timer0
      bclr    T5R             ; stop Timer5
      bclr    T6R             ; stop Timer6
      jmpa    cc_UC, MONITOR  ; go to the monitor


